#ifndef _TKWMCAPTURE
#define _TKWMCAPTURE


EXTERN int		WmCaptureReleaseCmd _ANSI_ARGS_((ClientData clientData,
			    Tcl_Interp *interp, int objc,
			    Tcl_Obj *CONST objv[]));
#endif  /* _TKWMCAPTURE */
